#ifndef _ADT_H
#define _ADT_H

/*{{{  includes */

#include <aterm1.h>
#include "ADT_dict.h"

/*}}}  */

/*{{{  prologue */


/*}}}  */
/*{{{  typedefs */

typedef struct _ADT_Entries *ADT_Entries;
typedef struct _ADT_Entry *ADT_Entry;
typedef struct _ADT_Separators *ADT_Separators;
typedef struct _ADT_Separator *ADT_Separator;

/*}}}  */

void ADT_initADTApi(void);

/*{{{  term conversion functions */

ADT_Entries ADT_EntriesFromTerm(ATerm t);
ATerm ADT_EntriesToTerm(ADT_Entries arg);
ADT_Entry ADT_EntryFromTerm(ATerm t);
ATerm ADT_EntryToTerm(ADT_Entry arg);
ADT_Separators ADT_SeparatorsFromTerm(ATerm t);
ATerm ADT_SeparatorsToTerm(ADT_Separators arg);
ADT_Separator ADT_SeparatorFromTerm(ATerm t);
ATerm ADT_SeparatorToTerm(ADT_Separator arg);

/*}}}  */
/*{{{  list functions */

int ADT_getEntriesLength (ADT_Entries arg);
ADT_Entries ADT_reverseEntries(ADT_Entries arg);
ADT_Entries ADT_appendEntries(ADT_Entries arg, ADT_Entry elem);
ADT_Entries ADT_concatEntries(ADT_Entries arg0, ADT_Entries arg1);
ADT_Entries ADT_sliceEntries(ADT_Entries arg, int start, int end);
ADT_Entry ADT_getEntriesEntryAt(ADT_Entries arg, int index);
ADT_Entries ADT_replaceEntriesEntryAt(ADT_Entries arg, ADT_Entry elem, int index);
ADT_Entries ADT_makeEntries1(ADT_Entry elem1);
ADT_Entries ADT_makeEntries2(ADT_Entry elem1, ADT_Entry elem2);
ADT_Entries ADT_makeEntries3(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3);
ADT_Entries ADT_makeEntries4(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4);
ADT_Entries ADT_makeEntries5(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4, ADT_Entry elem5);
ADT_Entries ADT_makeEntries6(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4, ADT_Entry elem5, ADT_Entry elem6);
int ADT_getSeparatorsLength (ADT_Separators arg);
ADT_Separators ADT_reverseSeparators(ADT_Separators arg);
ADT_Separators ADT_appendSeparators(ADT_Separators arg, ADT_Separator elem);
ADT_Separators ADT_concatSeparators(ADT_Separators arg0, ADT_Separators arg1);
ADT_Separators ADT_sliceSeparators(ADT_Separators arg, int start, int end);
ADT_Separator ADT_getSeparatorsSeparatorAt(ADT_Separators arg, int index);
ADT_Separators ADT_replaceSeparatorsSeparatorAt(ADT_Separators arg, ADT_Separator elem, int index);
ADT_Separators ADT_makeSeparators1(ADT_Separator elem1);
ADT_Separators ADT_makeSeparators2(ADT_Separator elem1, ADT_Separator elem2);
ADT_Separators ADT_makeSeparators3(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3);
ADT_Separators ADT_makeSeparators4(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4);
ADT_Separators ADT_makeSeparators5(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4, ADT_Separator elem5);
ADT_Separators ADT_makeSeparators6(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4, ADT_Separator elem5, ADT_Separator elem6);

/*}}}  */
/*{{{  constructors */

ADT_Entries ADT_makeEntriesMany(ADT_Entry head, ADT_Entries tail);
ADT_Entries ADT_makeEntriesEmpty();
ADT_Entry ADT_makeEntryConstructor(ATerm sort, ATerm alternative, ATerm termPattern);
ADT_Entry ADT_makeEntryList(ATerm sort, ATerm elemSort);
ADT_Entry ADT_makeEntrySeparatedList(ATerm sort, ATerm elemSort, ADT_Separators separators);
ADT_Separators ADT_makeSeparatorsMany(ADT_Separator head, ADT_Separators tail);
ADT_Separators ADT_makeSeparatorsEmpty();
ADT_Separator ADT_makeSeparatorDefault(ATerm termPattern);

/*}}}  */
/*{{{  equality functions */

ATbool ADT_isEqualEntries(ADT_Entries arg0, ADT_Entries arg1);
ATbool ADT_isEqualEntry(ADT_Entry arg0, ADT_Entry arg1);
ATbool ADT_isEqualSeparators(ADT_Separators arg0, ADT_Separators arg1);
ATbool ADT_isEqualSeparator(ADT_Separator arg0, ADT_Separator arg1);

/*}}}  */
/*{{{  ADT_Entries accessors */

ATbool ADT_isValidEntries(ADT_Entries arg);
inline ATbool ADT_isEntriesMany(ADT_Entries arg);
inline ATbool ADT_isEntriesEmpty(ADT_Entries arg);
ATbool ADT_hasEntriesHead(ADT_Entries arg);
ADT_Entry ADT_getEntriesHead(ADT_Entries arg);
ADT_Entries ADT_setEntriesHead(ADT_Entries arg, ADT_Entry head);
ATbool ADT_hasEntriesTail(ADT_Entries arg);
ADT_Entries ADT_getEntriesTail(ADT_Entries arg);
ADT_Entries ADT_setEntriesTail(ADT_Entries arg, ADT_Entries tail);

/*}}}  */
/*{{{  ADT_Entry accessors */

ATbool ADT_isValidEntry(ADT_Entry arg);
inline ATbool ADT_isEntryConstructor(ADT_Entry arg);
inline ATbool ADT_isEntryList(ADT_Entry arg);
inline ATbool ADT_isEntrySeparatedList(ADT_Entry arg);
ATbool ADT_hasEntrySort(ADT_Entry arg);
ATerm ADT_getEntrySort(ADT_Entry arg);
ADT_Entry ADT_setEntrySort(ADT_Entry arg, ATerm sort);
ATbool ADT_hasEntryAlternative(ADT_Entry arg);
ATerm ADT_getEntryAlternative(ADT_Entry arg);
ADT_Entry ADT_setEntryAlternative(ADT_Entry arg, ATerm alternative);
ATbool ADT_hasEntryTermPattern(ADT_Entry arg);
ATerm ADT_getEntryTermPattern(ADT_Entry arg);
ADT_Entry ADT_setEntryTermPattern(ADT_Entry arg, ATerm termPattern);
ATbool ADT_hasEntryElemSort(ADT_Entry arg);
ATerm ADT_getEntryElemSort(ADT_Entry arg);
ADT_Entry ADT_setEntryElemSort(ADT_Entry arg, ATerm elemSort);
ATbool ADT_hasEntrySeparators(ADT_Entry arg);
ADT_Separators ADT_getEntrySeparators(ADT_Entry arg);
ADT_Entry ADT_setEntrySeparators(ADT_Entry arg, ADT_Separators separators);

/*}}}  */
/*{{{  ADT_Separators accessors */

ATbool ADT_isValidSeparators(ADT_Separators arg);
inline ATbool ADT_isSeparatorsMany(ADT_Separators arg);
inline ATbool ADT_isSeparatorsEmpty(ADT_Separators arg);
ATbool ADT_hasSeparatorsHead(ADT_Separators arg);
ADT_Separator ADT_getSeparatorsHead(ADT_Separators arg);
ADT_Separators ADT_setSeparatorsHead(ADT_Separators arg, ADT_Separator head);
ATbool ADT_hasSeparatorsTail(ADT_Separators arg);
ADT_Separators ADT_getSeparatorsTail(ADT_Separators arg);
ADT_Separators ADT_setSeparatorsTail(ADT_Separators arg, ADT_Separators tail);

/*}}}  */
/*{{{  ADT_Separator accessors */

ATbool ADT_isValidSeparator(ADT_Separator arg);
inline ATbool ADT_isSeparatorDefault(ADT_Separator arg);
ATbool ADT_hasSeparatorTermPattern(ADT_Separator arg);
ATerm ADT_getSeparatorTermPattern(ADT_Separator arg);
ADT_Separator ADT_setSeparatorTermPattern(ADT_Separator arg, ATerm termPattern);

/*}}}  */
/*{{{  sort visitors */

ADT_Entries ADT_visitEntries(ADT_Entries arg, ADT_Entry (*acceptHead)(ADT_Entry));
ADT_Entry ADT_visitEntry(ADT_Entry arg, ATerm (*acceptSort)(ATerm), ATerm (*acceptAlternative)(ATerm), ATerm (*acceptTermPattern)(ATerm), ATerm (*acceptElemSort)(ATerm), ADT_Separators (*acceptSeparators)(ADT_Separators));
ADT_Separators ADT_visitSeparators(ADT_Separators arg, ADT_Separator (*acceptHead)(ADT_Separator));
ADT_Separator ADT_visitSeparator(ADT_Separator arg, ATerm (*acceptTermPattern)(ATerm));

/*}}}  */

#endif /* _ADT_H */
