#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "builtins.h"

/*{{{  typedefs */

typedef struct ATerm _D;

/*}}}  */

/*{{{  void initBuiltinsApi(void) */

void initBuiltinsApi(void)
{
  init_builtins_dict();
}

/*}}}  */

/*{{{  term conversion functions */

/*{{{  D DFromTerm(ATerm t) */

D DFromTerm(ATerm t)
{
  return (D)t;
}

/*}}}  */
/*{{{  ATerm DToTerm(D arg) */

ATerm DToTerm(D arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */


/*}}}  */
/*{{{  constructors */

/*{{{  D makeDInteger(int integer) */

D makeDInteger(int integer)
{
  return (D)(ATerm)ATmakeAppl1(afun0, (ATerm)ATmakeInt(integer));
}

/*}}}  */
/*{{{  D makeDDouble(float number) */

D makeDDouble(float number)
{
  return (D)(ATerm)ATmakeAppl1(afun1, (ATerm)ATmakeReal(number));
}

/*}}}  */
/*{{{  D makeDTrm(ATerm trm) */

D makeDTrm(ATerm trm)
{
  return (D)(ATerm)ATmakeAppl1(afun2, (ATerm)trm);
}

/*}}}  */
/*{{{  D makeDLst(ATermList lst) */

D makeDLst(ATermList lst)
{
  return (D)(ATerm)ATmakeAppl1(afun3, (ATerm)lst);
}

/*}}}  */
/*{{{  D makeDString(char* string) */

D makeDString(char* string)
{
  return (D)(ATerm)ATmakeAppl1(afun4, (ATerm)ATmakeAppl0(ATmakeAFun(string, 0, ATtrue)));
}

/*}}}  */
/*{{{  D makeDAll(int integer, float number, ATerm trm, char* string) */

D makeDAll(int integer, float number, ATerm trm, char* string)
{
  return (D)(ATerm)ATmakeAppl4(afun5, (ATerm)ATmakeInt(integer), (ATerm)ATmakeReal(number), (ATerm)trm, (ATerm)ATmakeAppl0(ATmakeAFun(string, 0, ATtrue)));
}

/*}}}  */
/*{{{  D makeDSome(int integer, float number) */

D makeDSome(int integer, float number)
{
  return (D)(ATerm)ATmakeAppl2(afun6, (ATerm)ATmakeInt(integer), (ATerm)ATmakeReal(number));
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool isEqualD(D arg0, D arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  D accessors */

/*{{{  ATbool isValidD(D arg) */

ATbool isValidD(D arg)
{
  if (isDInteger(arg)) {
    return ATtrue;
  }
  else if (isDDouble(arg)) {
    return ATtrue;
  }
  else if (isDTrm(arg)) {
    return ATtrue;
  }
  else if (isDLst(arg)) {
    return ATtrue;
  }
  else if (isDString(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  else if (isDSome(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isDInteger(D arg) */

inline ATbool isDInteger(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDInteger, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDDouble(D arg) */

inline ATbool isDDouble(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDDouble, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDTrm(D arg) */

inline ATbool isDTrm(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDTrm, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDLst(D arg) */

inline ATbool isDLst(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDLst, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDString(D arg) */

inline ATbool isDString(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDString, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDAll(D arg) */

inline ATbool isDAll(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDAll, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isDSome(D arg) */

inline ATbool isDSome(D arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternDSome, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasDInteger(D arg) */

ATbool hasDInteger(D arg)
{
  if (isDInteger(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  else if (isDSome(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  int getDInteger(D arg) */

int getDInteger(D arg)
{
  if (isDInteger(arg)) {
    return (int)ATgetInt((ATermInt)ATgetArgument((ATermAppl)arg, 0));
  }
  else if (isDAll(arg)) {
    return (int)ATgetInt((ATermInt)ATgetArgument((ATermAppl)arg, 0));
  }
  else 
    return (int)ATgetInt((ATermInt)ATgetArgument((ATermAppl)arg, 0));
}

/*}}}  */
/*{{{  D setDInteger(D arg, int integer) */

D setDInteger(D arg, int integer)
{
  if (isDInteger(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeInt(integer), 0);
  }
  else if (isDAll(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeInt(integer), 0);
  }
  else if (isDSome(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeInt(integer), 0);
  }

  ATabort("D has no Integer: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  ATbool hasDNumber(D arg) */

ATbool hasDNumber(D arg)
{
  if (isDDouble(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  else if (isDSome(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  float getDNumber(D arg) */

float getDNumber(D arg)
{
  if (isDDouble(arg)) {
    return (float)ATgetReal((ATermReal)ATgetArgument((ATermAppl)arg, 0));
  }
  else if (isDAll(arg)) {
    return (float)ATgetReal((ATermReal)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (float)ATgetReal((ATermReal)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  D setDNumber(D arg, float number) */

D setDNumber(D arg, float number)
{
  if (isDDouble(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeReal(number), 0);
  }
  else if (isDAll(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeReal(number), 1);
  }
  else if (isDSome(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeReal(number), 1);
  }

  ATabort("D has no Number: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  ATbool hasDTrm(D arg) */

ATbool hasDTrm(D arg)
{
  if (isDTrm(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATerm getDTrm(D arg) */

ATerm getDTrm(D arg)
{
  if (isDTrm(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
  }
  else 
    return (ATerm)ATgetArgument((ATermAppl)arg, 2);
}

/*}}}  */
/*{{{  D setDTrm(D arg, ATerm trm) */

D setDTrm(D arg, ATerm trm)
{
  if (isDTrm(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)trm, 0);
  }
  else if (isDAll(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)trm, 2);
  }

  ATabort("D has no Trm: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  ATbool hasDLst(D arg) */

ATbool hasDLst(D arg)
{
  if (isDLst(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATermList getDLst(D arg) */

ATermList getDLst(D arg)
{
  
    return (ATermList)ATgetArgument((ATermAppl)arg, 0);
}

/*}}}  */
/*{{{  D setDLst(D arg, ATermList lst) */

D setDLst(D arg, ATermList lst)
{
  if (isDLst(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)lst, 0);
  }

  ATabort("D has no Lst: %t\n", arg);
  return (D)NULL;
}

/*}}}  */
/*{{{  ATbool hasDString(D arg) */

ATbool hasDString(D arg)
{
  if (isDString(arg)) {
    return ATtrue;
  }
  else if (isDAll(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* getDString(D arg) */

char* getDString(D arg)
{
  if (isDString(arg)) {
    return (char*)ATgetName(ATgetAFun((ATermAppl)ATgetArgument((ATermAppl)arg, 0)));
  }
  else 
    return (char*)ATgetName(ATgetAFun((ATermAppl)ATgetArgument((ATermAppl)arg, 3)));
}

/*}}}  */
/*{{{  D setDString(D arg, char* string) */

D setDString(D arg, char* string)
{
  if (isDString(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeAppl0(ATmakeAFun(string, 0, ATtrue)), 0);
  }
  else if (isDAll(arg)) {
    return (D)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeAppl0(ATmakeAFun(string, 0, ATtrue)), 3);
  }

  ATabort("D has no String: %t\n", arg);
  return (D)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  D visitD(D arg, int (*acceptInteger)(int), float (*acceptNumber)(float), ATerm (*acceptTrm)(ATerm), ATermList (*acceptLst)(ATermList), char* (*acceptString)(char*)) */

D visitD(D arg, int (*acceptInteger)(int), float (*acceptNumber)(float), ATerm (*acceptTrm)(ATerm), ATermList (*acceptLst)(ATermList), char* (*acceptString)(char*))
{
  if (isDInteger(arg)) {
    return makeDInteger(
        acceptInteger ? acceptInteger(getDInteger(arg)) : getDInteger(arg));
  }
  if (isDDouble(arg)) {
    return makeDDouble(
        acceptNumber ? acceptNumber(getDNumber(arg)) : getDNumber(arg));
  }
  if (isDTrm(arg)) {
    return makeDTrm(
        acceptTrm ? acceptTrm(getDTrm(arg)) : getDTrm(arg));
  }
  if (isDLst(arg)) {
    return makeDLst(
        acceptLst ? acceptLst(getDLst(arg)) : getDLst(arg));
  }
  if (isDString(arg)) {
    return makeDString(
        acceptString ? acceptString(getDString(arg)) : getDString(arg));
  }
  if (isDAll(arg)) {
    return makeDAll(
        acceptInteger ? acceptInteger(getDInteger(arg)) : getDInteger(arg),
        acceptNumber ? acceptNumber(getDNumber(arg)) : getDNumber(arg),
        acceptTrm ? acceptTrm(getDTrm(arg)) : getDTrm(arg),
        acceptString ? acceptString(getDString(arg)) : getDString(arg));
  }
  if (isDSome(arg)) {
    return makeDSome(
        acceptInteger ? acceptInteger(getDInteger(arg)) : getDInteger(arg),
        acceptNumber ? acceptNumber(getDNumber(arg)) : getDNumber(arg));
  }
  ATabort("not a D: %t\n", arg);
  return (D)NULL;
}

/*}}}  */

/*}}}  */
